@@ -27,7 +27,6 @@ from keystone import exception
 from keystone.openstack.common import timeutils
 from keystone import tests
 from keystone.tests import default_fixtures
-from keystone.tests.ksfixtures import database
 from keystone import token
 from keystone import trust
 
@@ -69,22 +68,24 @@ def _build_user_auth(token=None, user_id=None, username=None,
 
 class AuthTest(tests.TestCase):
     def setUp(self):
-        self.useFixture(database.Database())
         super(AuthTest, self).setUp()
 
         self.load_backends()
         self.load_fixtures(default_fixtures)
 
+        # need to register the token provider first because auth controller
+        # depends on it
+        token.provider.Manager()
         self.context_with_remote_user = {'environment':
                                          {'REMOTE_USER': 'FOO',
                                           'AUTH_TYPE': 'Negotiate'}}
         self.empty_context = {'environment': {}}
 
         self.controller = token.controllers.Auth()
-        # This call sets up, among other things, the call to popen
-        # that will be used to run the CMS command.  These tests were
-        # passing only due to the global nature of the call.  If the
-        # tests in this file are run alone, API calls return unauthorized.
+        #This call sets up, among other things, the call to popen
+        #that will be used to run the CMS command.  These tests were
+        #passing only due to the global nature of the call.  If the
+        #tests in this file are run alone, API calls return unauthorized.
         environment.use_eventlet(monkeypatch_thread=False)
 
     def assertEqualTokens(self, a, b):
@@ -109,6 +110,9 @@ class AuthTest(tests.TestCase):
 
 
 class AuthBadRequests(AuthTest):
+    def setUp(self):
+        super(AuthBadRequests, self).setUp()
+
     def test_no_external_auth(self):
         """Verify that _authenticate_external() raises exception if N/A."""
         self.assertRaises(
@@ -197,6 +201,9 @@ class AuthBadRequests(AuthTest):
 
 
 class AuthWithToken(AuthTest):
+    def setUp(self):
+        super(AuthWithToken, self).setUp()
+
     def test_unscoped_token(self):
         """Verify getting an unscoped token with password creds."""
         body_dict = _build_user_auth(username='FOO',
@@ -266,8 +273,9 @@ class AuthWithToken(AuthTest):
         # Now create a group role for this user as well
         domain1 = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex}
         self.assignment_api.create_domain(domain1['id'], domain1)
-        new_group = {'domain_id': domain1['id'], 'name': uuid.uuid4().hex}
-        new_group = self.identity_api.create_group(new_group)
+        new_group = {'id': uuid.uuid4().hex, 'domain_id': domain1['id'],
+                     'name': uuid.uuid4().hex}
+        self.identity_api.create_group(new_group['id'], new_group)
         self.identity_api.add_user_to_group(self.user_foo['id'],
                                             new_group['id'])
         self.assignment_api.create_grant(
@@ -307,8 +315,9 @@ class AuthWithToken(AuthTest):
                                         role_group_domain1)
         self.assignment_api.add_user_to_project(project1['id'],
                                                 self.user_foo['id'])
-        new_group = {'domain_id': domain1['id'], 'name': uuid.uuid4().hex}
-        new_group = self.identity_api.create_group(new_group)
+        new_group = {'id': uuid.uuid4().hex, 'domain_id': domain1['id'],
+                     'name': uuid.uuid4().hex}
+        self.identity_api.create_group(new_group['id'], new_group)
         self.identity_api.add_user_to_group(self.user_foo['id'],
                                             new_group['id'])
         self.assignment_api.create_grant(
@@ -444,6 +453,9 @@ class AuthWithToken(AuthTest):
 
 
 class AuthWithPasswordCredentials(AuthTest):
+    def setUp(self):
+        super(AuthWithPasswordCredentials, self).setUp()
+
     def test_auth_invalid_user(self):
         """Verify exception is raised if invalid user."""
         body_dict = _build_user_auth(
@@ -521,15 +533,17 @@ class AuthWithPasswordCredentials(AuthTest):
 
         # 2) Create user "foo" in new domain with different password than
         #    default-domain foo.
+        new_user_id = uuid.uuid4().hex
         new_user_password = uuid.uuid4().hex
         new_user = {
+            'id': new_user_id,
             'name': self.user_foo['name'],
             'domain_id': new_domain_id,
             'password': new_user_password,
             'email': 'foo@bar2.com',
         }
 
-        new_user = self.identity_api.create_user(new_user)
+        self.identity_api.create_user(new_user_id, new_user)
 
         # 3) Update the default_domain_id config option to the new domain
 
@@ -547,6 +561,9 @@ class AuthWithPasswordCredentials(AuthTest):
 
 
 class AuthWithRemoteUser(AuthTest):
+    def setUp(self):
+        super(AuthWithRemoteUser, self).setUp()
+
     def test_unscoped_remote_authn(self):
         """Verify getting an unscoped token with external authn."""
         body_dict = _build_user_auth(
@@ -628,6 +645,7 @@ class AuthWithTrust(AuthTest):
     def setUp(self):
         super(AuthWithTrust, self).setUp()
 
+        trust.Manager()
         self.trust_controller = trust.controllers.TrustV3()
         self.auth_v3_controller = auth.controllers.Auth()
         self.trustor = self.user_foo
@@ -644,6 +662,10 @@ class AuthWithTrust(AuthTest):
                             'impersonation': True,
                             'roles': [{'id': self.role_browser['id']},
                                       {'name': self.role_member['name']}]}
+        expires_at = timeutils.strtime(timeutils.utcnow() +
+                                       datetime.timedelta(minutes=10),
+                                       fmt=TIME_FORMAT)
+        self.create_trust(expires_at=expires_at)
 
     def config_overrides(self):
         super(AuthWithTrust, self).config_overrides()
@@ -657,39 +679,34 @@ class AuthWithTrust(AuthTest):
                 'token_id': token_id,
                 'host_url': HOST_URL}
 
-    def create_trust(self, trust_data, trustor_name, expires_at=None,
-                     impersonation=True):
-        username = trustor_name
+    def create_trust(self, expires_at=None, impersonation=True):
+        username = self.trustor['name']
         password = 'foo2'
-        unscoped_token = self.get_unscoped_token(username, password)
-        context = self._create_auth_context(
-            unscoped_token['access']['token']['id'])
-        trust_data_copy = copy.deepcopy(trust_data)
-        trust_data_copy['expires_at'] = expires_at
-        trust_data_copy['impersonation'] = impersonation
-
-        return self.trust_controller.create_trust(
-            context, trust=trust_data_copy)['trust']
-
-    def get_unscoped_token(self, username, password='foo2'):
         body_dict = _build_user_auth(username=username, password=password)
-        return self.controller.authenticate({}, body_dict)
+        self.unscoped_token = self.controller.authenticate({}, body_dict)
+        context = self._create_auth_context(
+            self.unscoped_token['access']['token']['id'])
+        trust_data = copy.deepcopy(self.sample_data)
+        trust_data['expires_at'] = expires_at
+        trust_data['impersonation'] = impersonation
 
-    def build_v2_token_request(self, username, password, trust,
-                               tenant_id=None):
+        self.new_trust = self.trust_controller.create_trust(
+            context, trust=trust_data)['trust']
+
+    def build_v2_token_request(self, username, password, tenant_id=None):
         if not tenant_id:
             tenant_id = self.tenant_bar['id']
-        unscoped_token = self.get_unscoped_token(username, password)
-        unscoped_token_id = unscoped_token['access']['token']['id']
+        body_dict = _build_user_auth(username=username, password=password)
+        self.unscoped_token = self.controller.authenticate({}, body_dict)
+        unscoped_token_id = self.unscoped_token['access']['token']['id']
         request_body = _build_user_auth(token={'id': unscoped_token_id},
-                                        trust_id=trust['id'],
+                                        trust_id=self.new_trust['id'],
                                         tenant_id=tenant_id)
         return request_body
 
     def test_create_trust_bad_data_fails(self):
-        unscoped_token = self.get_unscoped_token(self.trustor['name'])
         context = self._create_auth_context(
-            unscoped_token['access']['token']['id'])
+            self.unscoped_token['access']['token']['id'])
         bad_sample_data = {'trustor_user_id': self.trustor['id'],
                            'project_id': self.tenant_bar['id'],
                            'roles': [{'id': self.role_browser['id']}]}
@@ -699,134 +716,100 @@ class AuthWithTrust(AuthTest):
                           context, trust=bad_sample_data)
 
     def test_create_trust_no_roles(self):
-        unscoped_token = self.get_unscoped_token(self.trustor['name'])
-        context = {'token_id': unscoped_token['access']['token']['id']}
+        context = {'token_id': self.unscoped_token['access']['token']['id']}
         self.sample_data['roles'] = []
         self.assertRaises(exception.Forbidden,
                           self.trust_controller.create_trust,
                           context, trust=self.sample_data)
 
     def test_create_trust(self):
-        expires_at = timeutils.strtime(timeutils.utcnow() +
-                                       datetime.timedelta(minutes=10),
-                                       fmt=TIME_FORMAT)
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'],
-                                      expires_at=expires_at)
-        self.assertEqual(self.trustor['id'], new_trust['trustor_user_id'])
-        self.assertEqual(self.trustee['id'], new_trust['trustee_user_id'])
+        self.assertEqual(self.trustor['id'], self.new_trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], self.new_trust['trustee_user_id'])
         role_ids = [self.role_browser['id'], self.role_member['id']]
-        self.assertTrue(timeutils.parse_strtime(new_trust['expires_at'],
+        self.assertTrue(timeutils.parse_strtime(self.new_trust['expires_at'],
                                                 fmt=TIME_FORMAT))
         self.assertIn('%s/v3/OS-TRUST/' % HOST_URL,
-                      new_trust['links']['self'])
+                      self.new_trust['links']['self'])
         self.assertIn('%s/v3/OS-TRUST/' % HOST_URL,
-                      new_trust['roles_links']['self'])
+                      self.new_trust['roles_links']['self'])
 
-        for role in new_trust['roles']:
+        for role in self.new_trust['roles']:
             self.assertIn(role['id'], role_ids)
 
     def test_create_trust_expires_bad(self):
         self.assertRaises(exception.ValidationTimeStampError,
-                          self.create_trust, self.sample_data,
-                          self.trustor['name'], expires_at="bad")
+                          self.create_trust,
+                          expires_at="bad")
         self.assertRaises(exception.ValidationTimeStampError,
-                          self.create_trust, self.sample_data,
-                          self.trustor['name'], expires_at="")
+                          self.create_trust,
+                          expires_at="")
         self.assertRaises(exception.ValidationTimeStampError,
-                          self.create_trust, self.sample_data,
-                          self.trustor['name'], expires_at="Z")
-
-    def test_create_trust_without_project_id(self):
-        """Verify that trust can be created without project id and
-        token can be generated with that trust.
-        """
-        unscoped_token = self.get_unscoped_token(self.trustor['name'])
-        context = self._create_auth_context(
-            unscoped_token['access']['token']['id'])
-        self.sample_data['project_id'] = None
-        self.sample_data['roles'] = []
-        new_trust = self.trust_controller.create_trust(
-            context, trust=self.sample_data)['trust']
-        self.assertEqual(self.trustor['id'], new_trust['trustor_user_id'])
-        self.assertEqual(self.trustee['id'], new_trust['trustee_user_id'])
-        self.assertIs(new_trust['impersonation'], True)
-        auth_response = self.fetch_v2_token_from_trust(new_trust)
-        token_user = auth_response['access']['user']
-        self.assertEqual(token_user['id'], new_trust['trustor_user_id'])
+                          self.create_trust,
+                          expires_at="Z")
 
     def test_get_trust(self):
-        unscoped_token = self.get_unscoped_token(self.trustor['name'])
-        context = {'token_id': unscoped_token['access']['token']['id'],
+        context = {'token_id': self.unscoped_token['access']['token']['id'],
                    'host_url': HOST_URL}
-        new_trust = self.trust_controller.create_trust(
-            context, trust=self.sample_data)['trust']
         trust = self.trust_controller.get_trust(context,
-                                                new_trust['id'])['trust']
+                                                self.new_trust['id'])['trust']
         self.assertEqual(self.trustor['id'], trust['trustor_user_id'])
         self.assertEqual(self.trustee['id'], trust['trustee_user_id'])
         role_ids = [self.role_browser['id'], self.role_member['id']]
-        for role in new_trust['roles']:
+        for role in self.new_trust['roles']:
             self.assertIn(role['id'], role_ids)
 
     def test_create_trust_no_impersonation(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'],
-                                      expires_at=None, impersonation=False)
-        self.assertEqual(self.trustor['id'], new_trust['trustor_user_id'])
-        self.assertEqual(self.trustee['id'], new_trust['trustee_user_id'])
-        self.assertIs(new_trust['impersonation'], False)
-        auth_response = self.fetch_v2_token_from_trust(new_trust)
+        self.create_trust(expires_at=None, impersonation=False)
+        self.assertEqual(self.trustor['id'], self.new_trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], self.new_trust['trustee_user_id'])
+        self.assertIs(self.new_trust['impersonation'], False)
+        auth_response = self.fetch_v2_token_from_trust()
         token_user = auth_response['access']['user']
-        self.assertEqual(token_user['id'], new_trust['trustee_user_id'])
+        self.assertEqual(token_user['id'], self.new_trust['trustee_user_id'])
 
         # TODO(ayoung): Endpoints
 
     def test_create_trust_impersonation(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        self.assertEqual(self.trustor['id'], new_trust['trustor_user_id'])
-        self.assertEqual(self.trustee['id'], new_trust['trustee_user_id'])
-        self.assertIs(new_trust['impersonation'], True)
-        auth_response = self.fetch_v2_token_from_trust(new_trust)
+        self.create_trust(expires_at=None)
+        self.assertEqual(self.trustor['id'], self.new_trust['trustor_user_id'])
+        self.assertEqual(self.trustee['id'], self.new_trust['trustee_user_id'])
+        self.assertIs(self.new_trust['impersonation'], True)
+        auth_response = self.fetch_v2_token_from_trust()
         token_user = auth_response['access']['user']
-        self.assertEqual(token_user['id'], new_trust['trustor_user_id'])
+        self.assertEqual(token_user['id'], self.new_trust['trustor_user_id'])
 
     def test_token_from_trust_wrong_user_fails(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        request_body = self.build_v2_token_request('FOO', 'foo2', new_trust)
-        self.assertRaises(exception.Forbidden, self.controller.authenticate,
-                          {}, request_body)
+        request_body = self.build_v2_token_request('FOO', 'foo2')
+        self.assertRaises(
+            exception.Forbidden,
+            self.controller.authenticate, {}, request_body)
 
     def test_token_from_trust_wrong_project_fails(self):
         for assigned_role in self.assigned_roles:
             self.assignment_api.add_role_to_user_and_project(
                 self.trustor['id'], self.tenant_baz['id'], assigned_role)
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        request_body = self.build_v2_token_request('TWO', 'two2', new_trust,
+        request_body = self.build_v2_token_request('TWO', 'two2',
                                                    self.tenant_baz['id'])
         self.assertRaises(exception.Forbidden, self.controller.authenticate,
                           {}, request_body)
 
-    def fetch_v2_token_from_trust(self, trust):
-        request_body = self.build_v2_token_request('TWO', 'two2', trust)
+    def fetch_v2_token_from_trust(self):
+        request_body = self.build_v2_token_request('TWO', 'two2')
         auth_response = self.controller.authenticate({}, request_body)
         return auth_response
 
-    def fetch_v3_token_from_trust(self, trust, trustee):
+    def fetch_v3_token_from_trust(self):
         v3_password_data = {
             'identity': {
                 "methods": ["password"],
                 "password": {
                     "user": {
-                        "id": trustee["id"],
-                        "password": trustee["password"]
-                    }
-                }
+                        "id": self.trustee["id"],
+                        "password": self.trustee["password"]}}
             },
             'scope': {
                 'project': {
-                    'id': self.tenant_baz['id']
-                }
-            }
-        }
+                    'id': self.tenant_baz['id']}}}
         auth_response = (self.auth_v3_controller.authenticate_for_token
                          ({'environment': {},
                            'query_string': {}},
@@ -838,7 +821,7 @@ class AuthWithTrust(AuthTest):
                 "methods": ["token"],
                 "token": {"id": token}},
             "scope": {
-                "OS-TRUST:trust": {"id": trust['id']}}}
+                "OS-TRUST:trust": {"id": self.new_trust['id']}}}
         token_auth_response = (self.auth_v3_controller.authenticate_for_token
                                ({'environment': {},
                                  'query_string': {}},
@@ -846,14 +829,13 @@ class AuthWithTrust(AuthTest):
         return token_auth_response
 
     def test_create_v3_token_from_trust(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        auth_response = self.fetch_v3_token_from_trust(new_trust, self.trustee)
+        auth_response = self.fetch_v3_token_from_trust()
 
         trust_token_user = auth_response.json['token']['user']
         self.assertEqual(self.trustor['id'], trust_token_user['id'])
 
         trust_token_trust = auth_response.json['token']['OS-TRUST:trust']
-        self.assertEqual(trust_token_trust['id'], new_trust['id'])
+        self.assertEqual(trust_token_trust['id'], self.new_trust['id'])
         self.assertEqual(self.trustor['id'],
                          trust_token_trust['trustor_user']['id'])
         self.assertEqual(self.trustee['id'],
@@ -863,8 +845,7 @@ class AuthWithTrust(AuthTest):
         self.assertEqual(2, len(trust_token_roles))
 
     def test_v3_trust_token_get_token_fails(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        auth_response = self.fetch_v3_token_from_trust(new_trust, self.trustee)
+        auth_response = self.fetch_v3_token_from_trust()
         trust_token = auth_response.headers['X-Subject-Token']
         v3_token_data = {'identity': {
             'methods': ['token'],
@@ -877,8 +858,7 @@ class AuthWithTrust(AuthTest):
              'query_string': {}}, v3_token_data)
 
     def test_token_from_trust(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        auth_response = self.fetch_v2_token_from_trust(new_trust)
+        auth_response = self.fetch_v2_token_from_trust()
 
         self.assertIsNotNone(auth_response)
         self.assertEqual(2,
@@ -886,23 +866,21 @@ class AuthWithTrust(AuthTest):
                          "user_foo has three roles, but the token should"
                          " only get the two roles specified in the trust.")
 
-    def assert_token_count_for_trust(self, trust, expected_value):
+    def assert_token_count_for_trust(self, expected_value):
         tokens = self.trust_controller.token_api._list_tokens(
-            self.trustee['id'], trust_id=trust['id'])
+            self.trustee['id'], trust_id=self.new_trust['id'])
         token_count = len(tokens)
         self.assertEqual(expected_value, token_count)
 
     def test_delete_tokens_for_user_invalidates_tokens_from_trust(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        self.assert_token_count_for_trust(new_trust, 0)
-        self.fetch_v2_token_from_trust(new_trust)
-        self.assert_token_count_for_trust(new_trust, 1)
+        self.assert_token_count_for_trust(0)
+        self.fetch_v2_token_from_trust()
+        self.assert_token_count_for_trust(1)
         self.token_api.delete_tokens_for_user(self.trustee['id'])
-        self.assert_token_count_for_trust(new_trust, 0)
+        self.assert_token_count_for_trust(0)
 
     def test_token_from_trust_cant_get_another_token(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
-        auth_response = self.fetch_v2_token_from_trust(new_trust)
+        auth_response = self.fetch_v2_token_from_trust()
         trust_token_id = auth_response['access']['token']['id']
         request_body = _build_user_auth(token={'id': trust_token_id},
                                         tenant_id=self.tenant_bar['id'])
@@ -911,12 +889,10 @@ class AuthWithTrust(AuthTest):
             self.controller.authenticate, {}, request_body)
 
     def test_delete_trust_revokes_token(self):
-        unscoped_token = self.get_unscoped_token(self.trustor['name'])
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
         context = self._create_auth_context(
-            unscoped_token['access']['token']['id'])
-        self.fetch_v2_token_from_trust(new_trust)
-        trust_id = new_trust['id']
+            self.unscoped_token['access']['token']['id'])
+        self.fetch_v2_token_from_trust()
+        trust_id = self.new_trust['id']
         tokens = self.token_api._list_tokens(self.trustor['id'],
                                              trust_id=trust_id)
         self.assertEqual(1, len(tokens))
@@ -926,26 +902,23 @@ class AuthWithTrust(AuthTest):
         self.assertEqual(0, len(tokens))
 
     def test_token_from_trust_with_no_role_fails(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
         for assigned_role in self.assigned_roles:
             self.assignment_api.remove_role_from_user_and_project(
                 self.trustor['id'], self.tenant_bar['id'], assigned_role)
-        request_body = self.build_v2_token_request('TWO', 'two2', new_trust)
+        request_body = self.build_v2_token_request('TWO', 'two2')
         self.assertRaises(
             exception.Forbidden,
             self.controller.authenticate, {}, request_body)
 
     def test_expired_trust_get_token_fails(self):
         expiry = "1999-02-18T10:10:00Z"
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'],
-                                      expiry)
-        request_body = self.build_v2_token_request('TWO', 'two2', new_trust)
+        self.create_trust(expiry)
+        request_body = self.build_v2_token_request('TWO', 'two2')
         self.assertRaises(
             exception.Forbidden,
             self.controller.authenticate, {}, request_body)
 
     def test_token_from_trust_with_wrong_role_fails(self):
-        new_trust = self.create_trust(self.sample_data, self.trustor['name'])
         self.assignment_api.add_role_to_user_and_project(
             self.trustor['id'],
             self.tenant_bar['id'],
@@ -954,32 +927,12 @@ class AuthWithTrust(AuthTest):
             self.assignment_api.remove_role_from_user_and_project(
                 self.trustor['id'], self.tenant_bar['id'], assigned_role)
 
-        request_body = self.build_v2_token_request('TWO', 'two2', new_trust)
+        request_body = self.build_v2_token_request('TWO', 'two2')
 
         self.assertRaises(
             exception.Forbidden,
             self.controller.authenticate, {}, request_body)
 
-    def test_do_not_consume_remaining_uses_when_get_token_fails(self):
-        trust_data = copy.deepcopy(self.sample_data)
-        trust_data['remaining_uses'] = 3
-        new_trust = self.create_trust(trust_data, self.trustor['name'])
-
-        for assigned_role in self.assigned_roles:
-            self.assignment_api.remove_role_from_user_and_project(
-                self.trustor['id'], self.tenant_bar['id'], assigned_role)
-
-        request_body = self.build_v2_token_request('TWO', 'two2', new_trust)
-        self.assertRaises(exception.Forbidden,
-                          self.controller.authenticate, {}, request_body)
-
-        unscoped_token = self.get_unscoped_token(self.trustor['name'])
-        context = self._create_auth_context(
-            unscoped_token['access']['token']['id'])
-        trust = self.trust_controller.get_trust(context,
-                                                new_trust['id'])['trust']
-        self.assertEqual(3, trust['remaining_uses'])
-
 
 class TokenExpirationTest(AuthTest):
 
