@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
 import sys
 
 import six
@@ -21,10 +22,10 @@ from keystone.common import dependency
 from keystone import config
 from keystone.contrib import federation
 from keystone import exception
-from keystone.i18n import _
-from keystone.openstack.common import jsonutils
+from keystone.openstack.common.gettextutils import _
 from keystone import token
 from keystone.token import provider
+from keystone import trust
 
 
 from keystone.openstack.common import log
@@ -43,7 +44,7 @@ class V2TokenDataHelper(object):
         metadata_ref = token_ref['metadata']
         if roles_ref is None:
             roles_ref = []
-        expires = token_ref.get('expires', provider.default_expire_time())
+        expires = token_ref.get('expires', token.default_expire_time())
         if expires is not None:
             if not isinstance(expires, six.text_type):
                 expires = timeutils.isotime(expires)
@@ -135,8 +136,8 @@ class V2TokenDataHelper(object):
 class V3TokenDataHelper(object):
     """Token data helper."""
     def __init__(self):
-        # Keep __init__ around to ensure dependency injection works.
-        super(V3TokenDataHelper, self).__init__()
+        if CONF.trust.enabled:
+            self.trust_api = trust.Manager()
 
     def _get_filtered_domain(self, domain_id):
         domain_ref = self.assignment_api.get_domain(domain_id)
@@ -240,7 +241,7 @@ class V3TokenDataHelper(object):
 
         if access_token:
             filtered_roles = []
-            authed_role_ids = jsonutils.loads(access_token['role_ids'])
+            authed_role_ids = json.loads(access_token['role_ids'])
             all_roles = self.assignment_api.list_roles()
             for role in all_roles:
                 for authed_role in authed_role_ids:
@@ -253,7 +254,7 @@ class V3TokenDataHelper(object):
         if CONF.trust.enabled and trust:
             token_user_id = trust['trustor_user_id']
             token_project_id = trust['project_id']
-            # trusts do not support domains yet
+            #trusts do not support domains yet
             token_domain_id = None
         else:
             token_user_id = user_id
@@ -305,15 +306,19 @@ class V3TokenDataHelper(object):
         if CONF.trust.enabled and trust:
             user_id = trust['trustor_user_id']
         if project_id or domain_id:
-            service_catalog = self.catalog_api.get_v3_catalog(
-                user_id, project_id)
+            try:
+                service_catalog = self.catalog_api.get_v3_catalog(
+                    user_id, project_id)
+            except exception.NotImplemented:
+                service_catalog = {}
+            # TODO(gyee): v3 service catalog is not quite completed yet
             # TODO(ayoung): Enforce Endpoints for trust
             token_data['catalog'] = service_catalog
 
     def _populate_token_dates(self, token_data, expires=None, trust=None,
                               issued_at=None):
         if not expires:
-            expires = provider.default_expire_time()
+            expires = token.default_expire_time()
         if not isinstance(expires, six.string_types):
             expires = timeutils.isotime(expires, subsecond=True)
         token_data['expires_at'] = expires
@@ -359,24 +364,11 @@ class V3TokenDataHelper(object):
 class BaseProvider(provider.Provider):
     def __init__(self, *args, **kwargs):
         super(BaseProvider, self).__init__(*args, **kwargs)
+        if CONF.trust.enabled:
+            self.trust_api = trust.Manager()
         self.v3_token_data_helper = V3TokenDataHelper()
         self.v2_token_data_helper = V2TokenDataHelper()
 
-    def _create_token(self, token_id, token_data):
-        try:
-            if isinstance(token_data['expires'], six.string_types):
-                token_data['expires'] = timeutils.normalize_time(
-                    timeutils.parse_isotime(token_data['expires']))
-            self.token_api.create_token(token_id, token_data)
-        except Exception:
-            exc_info = sys.exc_info()
-            # an identical token may have been created already.
-            # if so, return the token_data as it is also identical
-            try:
-                self.token_api.get_token(token_id)
-            except exception.TokenNotFound:
-                six.reraise(*exc_info)
-
     def get_token_version(self, token_data):
         if token_data and isinstance(token_data, dict):
             if 'token_version' in token_data:
@@ -390,7 +382,7 @@ class BaseProvider(provider.Provider):
                 return token.provider.V2
             if 'token' in token_data and 'methods' in token_data['token']:
                 return token.provider.V3
-        raise exception.UnsupportedTokenVersionException()
+        raise token.provider.UnsupportedTokenVersionException()
 
     def issue_v2_token(self, token_ref, roles_ref=None,
                        catalog_ref=None):
@@ -398,18 +390,30 @@ class BaseProvider(provider.Provider):
             token_ref, roles_ref, catalog_ref)
         token_id = self._get_token_id(token_data)
         token_data['access']['token']['id'] = token_id
-        expiry = token_data['access']['token']['expires']
-        data = dict(key=token_id,
-                    id=token_id,
-                    expires=expiry,
-                    user=token_ref['user'],
-                    tenant=token_ref['tenant'],
-                    metadata=token_ref['metadata'],
-                    token_data=token_data,
-                    bind=token_ref.get('bind'),
-                    trust_id=token_ref['metadata'].get('trust_id'),
-                    token_version=token.provider.V2)
-        self._create_token(token_id, data)
+        try:
+            expiry = token_data['access']['token']['expires']
+            if isinstance(expiry, six.string_types):
+                expiry = timeutils.normalize_time(
+                    timeutils.parse_isotime(expiry))
+            data = dict(key=token_id,
+                        id=token_id,
+                        expires=expiry,
+                        user=token_ref['user'],
+                        tenant=token_ref['tenant'],
+                        metadata=token_ref['metadata'],
+                        token_data=token_data,
+                        bind=token_ref.get('bind'),
+                        trust_id=token_ref['metadata'].get('trust_id'),
+                        token_version=token.provider.V2)
+            self.token_api.create_token(token_id, data)
+        except Exception:
+            exc_info = sys.exc_info()
+            # an identical token may have been created already.
+            # if so, return the token_data as it is also identical
+            try:
+                self.token_api.get_token(token_id)
+            except exception.TokenNotFound:
+                raise exc_info[0], exc_info[1], exc_info[2]
 
         return (token_id, token_data)
 
@@ -448,31 +452,42 @@ class BaseProvider(provider.Provider):
             access_token=access_token)
 
         token_id = self._get_token_id(token_data)
-
-        expiry = token_data['token']['expires_at']
-        # FIXME(gyee): is there really a need to store roles in metadata?
-        role_ids = []
-        if metadata_ref is None:
-            metadata_ref = {}
-        if 'project' in token_data['token']:
-            # project-scoped token, fill in the v2 token data
-            # all we care are the role IDs
-            role_ids = [r['id'] for r in token_data['token']['roles']]
-            metadata_ref = {'roles': role_ids}
-        if trust:
-            metadata_ref.setdefault('trust_id', trust['id'])
-            metadata_ref.setdefault('trustee_user_id',
-                                    trust['trustee_user_id'])
-        data = dict(key=token_id,
-                    id=token_id,
-                    expires=expiry,
-                    user=token_data['token']['user'],
-                    tenant=token_data['token'].get('project'),
-                    metadata=metadata_ref,
-                    token_data=token_data,
-                    trust_id=trust['id'] if trust else None,
-                    token_version=token.provider.V3)
-        self._create_token(token_id, data)
+        try:
+            expiry = token_data['token']['expires_at']
+            if isinstance(expiry, six.string_types):
+                expiry = timeutils.normalize_time(
+                    timeutils.parse_isotime(expiry))
+            # FIXME(gyee): is there really a need to store roles in metadata?
+            role_ids = []
+            if metadata_ref is None:
+                metadata_ref = {}
+            if 'project' in token_data['token']:
+                # project-scoped token, fill in the v2 token data
+                # all we care are the role IDs
+                role_ids = [r['id'] for r in token_data['token']['roles']]
+                metadata_ref = {'roles': role_ids}
+            if trust:
+                metadata_ref.setdefault('trust_id', trust['id'])
+                metadata_ref.setdefault('trustee_user_id',
+                                        trust['trustee_user_id'])
+            data = dict(key=token_id,
+                        id=token_id,
+                        expires=expiry,
+                        user=token_data['token']['user'],
+                        tenant=token_data['token'].get('project'),
+                        metadata=metadata_ref,
+                        token_data=token_data,
+                        trust_id=trust['id'] if trust else None,
+                        token_version=token.provider.V3)
+            self.token_api.create_token(token_id, data)
+        except Exception:
+            exc_info = sys.exc_info()
+            # an identical token may have been created already.
+            # if so, return the token_data as it is also identical
+            try:
+                self.token_api.get_token(token_id)
+            except exception.TokenNotFound:
+                raise exc_info[0], exc_info[1], exc_info[2]
 
         return (token_id, token_data)
 
@@ -664,4 +679,4 @@ class BaseProvider(provider.Provider):
             return self._validate_v3_token_ref(token_ref)
         elif version == token.provider.V2:
             return self._validate_v2_token_ref(token_ref)
-        raise exception.UnsupportedTokenVersionException()
+        raise token.provider.UnsupportedTokenVersionException()
